/**********************************************************************
 * options.c                                            Sepmtember 1999
 * Horms                                             horms@valinux.com.
 * VA Linux System                                 http://valinux.com./
 *
 * Read in command line options
 * Code based on man getopt(3)
 *
 * Released under the terms of the GNU GPL
 **********************************************************************/

#include "options.h"

int idle_timeout;


/**********************************************************************
 * options
 * Read in command line options
 **********************************************************************/

options_t options(int argc, char **argv){
  int c;
  options_t options;
  char * short_options_string="acm:ip:P:u:g:hjt:";

  extern int idle_timeout;

  options.client_server_specification=0;
  options.authenticate_in=0;
  options.mapname=DEFAULT_MAPNAME;
  options.inetd_mode=0;
  options.port=PERDITION_PORT_PROTOCOL_DEPENDANT;
  options.username=DEFAULT_USERNAME;
  options.group=DEFAULT_GROUP;
  options.protocol=DEFAULT_PROTOCOL;
  idle_timeout=DEFAULT_IDLE_TIMEOUT;

  while (1){
    int option_index = 0;
#ifdef HAVE_GETOPT_LONG
    static struct option long_options[] =
    {
      {"map_name", 1, 0, 0},
      {"inetd_mode", 0, 0, 0},
      {"client_server_specification", 0, 0, 0},
      {"authenticate_in", 0, 0, 0},
      {"port", 1, 0, 0},
      {"username", 1, 0, 0},
      {"group", 1, 0, 0},
      {"help", 0, 0, 0},
      {"protocol", 1, 0, 0},
      {"timeout", 1, 0, 0},
      {"jain", 2, 0, 0},
      {"jane", 2, 0, 0},
      {"jayne", 2, 0, 0},
      {0, 0, 0, 0}
    };

    c = getopt_long (
      argc, 
      argv, 
      short_options_string, 
      long_options, 
      &option_index
    );
#else
    c = getopt (argc, argv, short_options_string);
#endif
    if (c == -1){
      return(options);
    }

    switch (c){
#ifdef HAVE_GETOPT_LONG
      case 0:
        if(strcmp(long_options[option_index].name,"help")==0){
          usage(0);
	  break;
        }
        if(str_is_jain(long_options[option_index].name)){
          printf(
            "%s, Oath\n", 
            long_options[option_index].name
          );
          break;
        }
        if(strcmp(long_options[option_index].name,"map_name")==0){
          options.mapname=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"inetd_mode")==0){
          options.inetd_mode=1;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "client_server_specification"
	)==0){
          options.client_server_specification=1;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "authenticate_in"
	)==0){
          options.authenticate_in=1;
          break;
        }
        if(strcmp(long_options[option_index].name,"port")==0){
          options.port=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"protocol")==0){
          if((options.protocol=protocol_index(optarg))<0){
            syslog(LOG_WARNING, "options: invalid protocol: %s", optarg);
            usage(-1);
          }
          break;
        }
        if(strcmp(long_options[option_index].name,"username")==0){
          options.username=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"group")==0){
          options.group=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"idle_timeout")==0){
          if(!str_is_digit(optarg)){
            usage(-1);
          }
          idle_timeout=atoi(optarg);
          break;
        }
        break;
#endif
      case 'h':
        usage(0);
      case 'j':
        printf("Jain, Oath\n");
        break;
      case 'm':
        options.mapname=optarg;
        break;
      case 'i':
        options.inetd_mode=1;
        break;
      case 'c':
        options.client_server_specification=1;
        break;
      case 'a':
        options.authenticate_in=1;
        break;
      case 'p':
        options.port=optarg;
        break;
      case 'P':
        if((options.protocol=protocol_index(optarg))<0){
          syslog(LOG_WARNING, "options: invalid protocol: %s", optarg);
          usage(-1);
        }
        break;
      case 'u':
        options.username=optarg;
        break;
      case 'g':
        options.group=optarg;
        break;
      case 't':
        if(!str_is_digit(optarg)){
          usage(-1);
        }
        idle_timeout=atoi(optarg);
        break;
      case '?':
        usage(-1);
      default:
        printf ("?? getopt returned character code 0%o ??\n", c);
      }
  }

  if (optind<argc) {
    usage(-1);
  }

  return(options);
}


/**********************************************************************
 * log_options
 * Log options to syslog
 **********************************************************************/

int log_options(options_t options){
  char *protocol;

  extern int idle_timeout;
  extern struct utsname *system_uname;

  if((protocol=protocol_list(protocol, NULL, options.protocol))==NULL){
    syslog(LOG_WARNING, "log_options: protocol_list");
    return(-1);
  }

  syslog(
    LOG_INFO, 
    "map_name=\"%s\", inetd_mode=%d, port=\"%s\", timeout=%d, "
    "username=\"%s\", group=\"%s\" prototol=\"%s\", "
    "server_port_specification=%d,  nodename=\"%s\", "
    "authenticate_in=%d\n",
    options.mapname,
    options.inetd_mode,
    options.port,
    idle_timeout,
    options.username,
    options.group,
    protocol,
    options.client_server_specification,
    system_uname->nodename,
    options.authenticate_in
  );

  free(protocol);
  return(0);
}


/**********************************************************************
 * usage
 * Display usage unfromtaion
 * Printed to stdout if exit_status=0, stderr otherwise
 **********************************************************************/

void usage(int exit_status){
  FILE *stream;
  char *available_protocols;
  char *default_protocol_str;

  if(exit_status!=0){
     stream=stderr;
  }
  else{
     stream=stdout;
  }
  
  if((available_protocols=protocol_list(
    available_protocols, 
    ", ", 
    PROTOCOL_ALL
  ))==NULL){
    syslog(LOG_WARNING, "usage: protocol_list 1");
    available_protocols="*error*";
  }

  if((default_protocol_str=protocol_list(
    default_protocol_str, 
    NULL,
    PROTOCOL_DEFAULT
  ))==NULL){
    syslog(LOG_WARNING, "usage: protocol_list 2");
    default_protocol_str="*error*";
  }

  fprintf(
    stream, 
    "Usage: perdition [options]\n"
    "  options:\n"
#ifdef HAVE_GETOPT_LONG    
    "     -a|--authenticate_in: \n"
    "                      User is authenticated by perdition before\n"
    "                      connection to backend server is made\n"
    "     -c|--client_server_specification: \n"
    "                      Allow USER of the form user%cserver[:port] to\n"
    "                      specify the server and port for a user\n"
    "     -g|--group:      Group to run as (default %s)\n"
    "     -h|--help:       Display this message\n"
    "     -i|--inetd_mode: Run in inetd mode\n"
    "     -m|--map_name    gdbm map for users to hosts and port\n"
    "                        (default %s)\n"
    "     -P|--protocol:   Protocol to use (default %s)\n"
    "                      available protocols: %s\n"
    "     -p|--port:       Port to listen on (default %s)\n"
    "     -t|--timeout:    Idle timeout in seconds (default %d)\n"
    "     -u|--username:   Username to run as (default %s)\n",
#else
    "     -a  User is authenticated by perdition before\n"
    "         connection to backend server is made\n"
    "     -c: Allow USER of the form user%cserver[:port] to\n"
    "         specify the server and port for a user\n"
    "     -g: Group to run as (default %s)\n"
    "     -h: Display this message\n"
    "     -i: Run in inetd mode\n"
    "     -m: gdbm map for users to hosts and port\n"
    "         (default %s)\n"
    "     -P: Protocol to use (default %s)\n"
    "         available protocols: %s\n"
    "     -p: Port to listen on (default %s)\n"
    "     -t: Idle timeout in seconds (default %d)\n"
    "     -u: Username to run as (default %s)\n",
#endif
    CLIENT_SERVER_DELIMITER,
    DEFAULT_GROUP,
    DEFAULT_MAPNAME,
    default_protocol_str,
    available_protocols,
    PERDITION_PORT_PROTOCOL_DEPENDANT,
    DEFAULT_IDLE_TIMEOUT,
    DEFAULT_USERNAME
  );

  daemon_exit_cleanly(exit_status);
}
