/**********************************************************************
 * perdition_gdbm.c                                      September 1999
 * Horms                                             horms@valinux.com.
 * VA Linux System                                 http://valinux.com./
 *
 * Access a gdbm(3) database
 *
 * Note: If you wish to support other databases, such as a RDBMS,
 *       other dbm formats or a flat file please make an 
 *       abstraction layer, posibly in the same vane as the protocol
 *       abstraction used to so far support both POP3 and IMAP4.
 *       Please contact me if you are working on this as I am
 *       very keen to get this done.
 *
 * Client server specification code courtesy of Daniel Rosen,
 * <droesen@entire-systems.com>. 
 *
 * Released under the terms of the GNU GPL
 **********************************************************************/

#include "perdition_gdbm.h"


/**********************************************************************
 * gdbm_getserver
 * Read the server information for a given key from the gdbmmap
 * specified in the options structure.
 * Result is a pointer to a server_port_t structure that is
 * allocated by this function
 **********************************************************************/

server_port_t *gdbm_getserver(
  const options_t options,
  char * key_str
){
  GDBM_FILE dbf;
  datum key;
  datum content;
  server_port_t *server_port;
  char *popserver;
  
  if (
    options.client_server_specification &&
    (popserver=strchr(key_str, CLIENT_SERVER_DELIMITER)) != NULL 
  ){
    content.dptr = strdup(popserver+1);
    content.dsize = strlen(content.dptr);
    *popserver = '\0';
  }
  else{
    key.dptr=key_str;
    key.dsize=strlen(key_str);

    if((dbf=gdbm_open(options.mapname, 0, GDBM_READER, 0644, 0))==NULL){
      syslog(
        LOG_WARNING, 
        "gdbm_getserver: gdbm_open: %s: %s", 
        gdbm_strerror(gdbm_errno), 
        strerror(errno)
      );
      return(NULL);
    }
    content=gdbm_fetch(dbf,key);
    gdbm_close(dbf);
  }

  if((server_port=server_port_create(server_port))==NULL){
    syslog(LOG_WARNING, "gdbm_getserver: server_port_create");
    return(NULL);
  }
  
  if(content.dptr!=NULL){
    server_port=server_port_datum_assign(server_port, content);
  }
  else{
    return(NULL);
  }

  return(server_port);
} 
